import lxml.html
import requests
import tempfile
import os
import sys
import shutil
import pygments.lexers
import pygments.util
import subprocess
import json
import collections
import time

from werkzeug.contrib.cache import SimpleCache

#from oraculum.config import EASYFIX_CACHE_PROJECTINFO, EASYFIX_CACHE_TICKETS
from oraculum.utils import Action, ACTION_TIMES


INFO = {
        'provider': 'easyfix',
        'name': 'Easyfix',
        'duration': ACTION_TIMES['couple of days'],
        'description': 'Number of projects within Fedora which are looking for help. These tasks have been evaluated as easy entrance point.',
        'scope': 'Programming',
        'tags': True,
       }

def parse_easyfix_page():
    q = requests.get("https://fedoraproject.org/easyfix/index.html")
    projects = lxml.html.fromstring(q.text).xpath("//div[@id='tabs-1']/div[@class='main-page-section']")

    data = []

    for p in projects:
        header = p.xpath("./h2/a")[0]
        #contact = p.xpath("./p")[0]
        tickets = p.xpath("./p")[1:]

        name = header.attrib['name'].strip()
        url = header.attrib['href'].strip()
        data.append({"name": name, "url": url, "tickets": []})

        for t in tickets:
            title = t.text_content().strip().split('\n')[1].strip()
            url = t.xpath("./a")[0].attrib['href'].strip()
            # FIXME - add code for grabbing description off of the ticket
            data[-1]['tickets'].append({"title": title, "url": url, "description": None})

    return data

def clone_repo(repo):
    """Clones the repo.
    Returns full path to the cloned repo, and the cloned branch."""

    fs_dir= tempfile.mkdtemp(suffix='_easyfix_pygments')
    repo_dir = os.path.join(fs_dir, 'repo')

    # for now, just delete the repo and make a new shallow copy
    if os.path.exists(repo_dir):
        shutil.rmtree(repo_dir)

    cmd = ['git', 'clone', '--depth', '1', repo, repo_dir]

    try:
        subprocess.check_output(cmd, stderr=subprocess.STDOUT)
    except subprocess.CalledProcessError as e:
        print("Could not clone repo %s" % repo)
        return None, None

    return repo_dir, fs_dir


def score_lexers(repo_dir):
    lexers = {}
    running_score = [0, 0]
    for root, dirs, files in os.walk(repo_dir, topdown=True, followlinks=False):
        # ignore 'hidden' files and symlinks
        files = [f for f in files if not f[0] == '.' and not os.path.islink(os.path.join(root, f))]
        # Note the dirs[:] = slice assignment;
        #  we are replacing the elements in dirs (and not the list referred to by dirs)
        #  so that os.walk() will not process deleted directories (works with topdown=True)
        dirs[:] = [d for d in dirs if not d[0] == '.']
        for f in files:
            try:
                lexer = pygments.lexers.guess_lexer_for_filename(f, '').name
            except pygments.util.ClassNotFound:
#                try:
#                    lexer = pygments.lexers.guess_lexer(open(os.path.join(root, f)).read(1024)).name + " __ text"
#                except (pygments.util.ClassNotFound, UnicodeDecodeError):
#                    lexer = 'Unknown'
                continue

            score = lexers.get(lexer, [0, 0])
            score[0] += os.path.getsize(os.path.join(root, f))
            score[1] += 1
            running_score[0] += score[0]
            running_score[1] += score[1]
            lexers[lexer] = score[:]
    return collections.OrderedDict([(l, lexers[l]) for l in sorted(lexers, key=lexers.get, reverse=True)])


def grab_project_metadata(url):
    repo_dir, fsdir = clone_repo(url)
    if not repo_dir:
        return None
    scores = score_lexers(repo_dir)
    shutil.rmtree(fsdir)

    return {"content_types": scores}


def filter_content_types(cts):
    ignore = set([
                'Debian Control file', 'SquidConf', 'XML', 'YAML', 'Gettext Catalog', 'LessCss',
                'reStructuredText', 'JSON', 'RPMSpec', 'XSLT', 'Javascript+mozpreproc', 'markdown',
                'Standard ML', 'SCSS', 'Smarty', 'Makefile', 'Cheetah', 'Nginx configuration file',
                'INI', 'Text only', 'Batchfile', 'Groff', 'ApacheConf', 'DTD', 'Diff', 'Docker',
                'CSS',
                ])
    return list(set(cts.keys()) - ignore)

IGNORE = [
        'fedora-commops', 'fedora-diversity', 'fedora-infrastructure', 'fedora-docs/docs-fp-o', 'fedora-docs/quick-docs',
        'fedora-hubs', 'fedora-join/Fedora-Join', 'fedora-qa'
        ]


PROJINFO_DB = {
    'RITlug/TigerOS': {'description': 'Fedora Remix maintained by the Rochester Institute of Technology (RIT) Linux Users '
                       'Group, targeted at users new to Linux and RIT students, faculty, and staff ',
                       'languages': ['Ruby', 'Bash']},
    'RITlug/teleirc': {'description': 'NodeJS implementation of a Telegram <=> IRC bridge for use with any IRC channel '
                       'and Telegram group',
                       'languages': ['JavaScript']},
    'SSSD/sssd': {'description': 'SSSD provides a set of daemons to manage access to remote directories and '
                  'authentication mechanisms such as LDAP, Kerberos or FreeIPA.',
                  'languages': ['C', 'Python', 'D']},
    'fedocal': {'description': 'fedocal is a web based calendar application.', 'languages': ['Python', 'JavaScript']},
    'fedora-infra/bodhi': {'description': 'Bodhi is a web-system that facilitates the process of publishing updates for a '
                           'Fedora-based software distribution.',
                           'languages': ['Python']},
    'fedora-infra/fas': {'description': 'The Fedora Account System is a community oriented accounts system which aims to '
                         'provide a self-driven and self-controled management to its registered users.',
                         'languages': ['JavaScript', 'HTML', 'Python']},
    'fedora-infra/fedmsg': {'description': 'fedmsg (Federated Message Bus) is a library built on ZeroMQ using the PyZMQ '
                            'Python bindings. fedmsg aims to make it easy to connect services together '
                            'using ZeroMQ publishers and subscribers.',
                            'languages': ['Python']},
    'fedora-infra/fedora-packages': {'description': 'A webapp that allows searching packages in Fedora. Written in Python '
                                     'using TurboGears2 and Moksha.',
                                     'languages': ['JavaScript', 'Python']},
    'fedora-infra/fedora-tagger': {'description': 'Flask app for helping us tag Fedora packages.',
                                   'languages': ['Python', 'JavaScript']},
    'fedora-infra/fmn.consumer': {'description': 'fmn is a family of systems to manage end-user notifications triggered '
                                  'by fedmsg, the FEDerated MESsage bus. fmn provides a single place for '
                                  'all applications using fedmsg to notify users of events. Notifications '
                                  'can be delivered by email, irc, and server-sent events. Users can '
                                  'configure their notifications for all the applications they use in one '
                                  'place.',
                                  'languages': ['Python']},
    'fedora-infra/fmn.web': {'description': 'fmn is a family of systems to manage end-user notifications triggered by '
                             'fedmsg, the FEDerated MESsage bus. fmn provides a single place for all '
                             'applications using fedmsg to notify users of events. Notifications can be '
                             'delivered by email, irc, and server-sent events. Users can configure their '
                             'notifications for all the applications they use in one place.',
                             'languages': ['HTML', 'Python']},
    'fedora-infra/pkgdb2': {'description': 'PackageDB is the application handling who is allowed to commit on the git of '
                            'the Fedora packages, it also handles who is the person getting the bugs on '
                            'the bugzilla and who get the notifications for changes in the git, builds or '
                            'bugs.',
                            'languages': ['JavaScript', 'Python']},
    'fedora-qa/os-autoinst-distri-fedora': {'description': 'openQA tests for the Fedora distribution',
                                            'languages': ['Perl']},
    'fedora-qa/testdays-web': {'description': 'Simple Flask web-app for reporting Fedora test days results.',
                               'languages': ['HTML', 'Python']},
    'fedora-websites': {'description': 'This is the Fedora Websites GIT repo used to build the websites starting from the '
                        'F24 release cycle.',
                        'languages': ['HTML', 'Python', 'Bash', 'PHP', 'JavaScript', 'C#']},
    'freeipa': {'description': 'FreeIPA is an integrated Identity and Authentication solution for Linux/UNIX networked '
                'environments.',
                'languages': ['Python', 'Bash', 'C']},
    'ipsilon': {'description': 'The Ipsilon project implements an Identity Provider that is easy to use and configure.',
                'languages': ['Python']},
    'kushaldas/autocloud': {'description': 'In simple words this services listens to fedmsg messages for successful koji '
                            'builds of cloud base, and atomic images. When found, it downloads those '
                            'images, and test them locally using Tunir.',
                            'languages': ['HTML', 'JavaScript', 'Python']},
    'pagure': {'description': 'With pagure you can host your project with its documentation, let your users report issues '
               'or request enhancements using the ticketing system and build your community of '
               'contributors by allowing them to fork your projects and contribute to it via the '
               'now-popular pull-request mechanism.',
               'languages': ['HTML', 'JavaScript', 'Python']},
    'pungi': {'description': 'Distribution compose tool', 'languages': ['Python']},
    'release-monitoring/anitya': {'description': 'Anitya is a release monitoring project. It provides a user-friendly '
                                  'interface to add, edit, or browse projects.',
                                  'languages': ['Python', 'JavaScript']},
    'taskotron/libtaskotron': {'description': 'libtaskotron is a library for running automated tasks as part of the '
                               'Taskotron system. The initial objective is (but is not limited to) '
                               'automating selected package checks in Fedora. ',
                               'languages': ['Python']},
    'taskotron/task-rpmlint': {'description': 'Taskotron task to run rpmlint on all new Koji builds',
                               'languages': ['Python']}}

EASYFIX_CACHE = SimpleCache()

def get_actions(): # FIXME - store/load the projects data for realz
    actions  = EASYFIX_CACHE.get('easyfix_actions')
    if actions is not None:
        return actions

    actions = []
    print("Parsing Easyfix page")
    projects = parse_easyfix_page()


    for p in projects:
        if p['name'] in IGNORE:
            continue
        if p['name'] not in PROJINFO_DB:
            print("MISS on %s - %s" % (p['name'], p['url']))
            continue

        for t in p['tickets']:
            tags = PROJINFO_DB[p['name']]['languages'][:]
            project_org = p['name'].split('/', 1)[0]
            project_name = p['name'].split('/', 1)[-1]
            actions.append(Action(
                name=t['title'],
                tags=tags,
                duration=ACTION_TIMES['couple of days'],
                provider=INFO['provider'],
                extra_data={
                    "ticket_url": t['url'],
                    'repo_org': project_org,
                    'repo_name': project_name,
                    'repo_fullname': p['name'],
                    'repo_url': p['url'],
                    'description': PROJINFO_DB[p['name']]['description'],
                    'languages': PROJINFO_DB[p['name']]['languages'],
                  },
                )
              )

    EASYFIX_CACHE.set('easyfix_actions', actions, timeout=3600)
    return actions


#def get_actions():
#    actions = []
#    project_info = {}
#    projects = {}
#    if os.path.isfile(EASYFIX_CACHE_PROJECTINFO):
#        with open(EASYFIX_CACHE_PROJECTINFO) as f:
#            project_info = json.load(f)
#    if os.path.isfile(EASYFIX_CACHE_TICKETS) and (time.time() - os.stat(EASYFIX_CACHE_TICKETS).st_mtime) < 3600: # FIXME
#        with open(EASYFIX_CACHE_TICKETS) as f:
#            projects = json.load(f)
#    else:
#        print("Parsing Easyfix page")
#        projects = parse_easyfix_page()
#        with open(EASYFIX_CACHE_TICKETS, 'w') as f:
#            json.dump(projects, f)
#
#    for p in projects:
#        if p['url'] in project_info:
#            p.update(project_info[p['url']])
#        else:
#            if p['name'] in IGNORE:
#                continue
#            print("Cloning project", p['name'], p['url'])
#            data = grab_project_metadata(p['url'])
#            if data is None:
#                continue
#            p.update(data)
#            project_info[p['url']] = data
#        for t in p['tickets']:
#            tags=filter_content_types(p['content_types'])
#            if not tags:
#                continue
#            tags.append('Programming')
#            project_org = p['name'].split('/', 1)[0]
#            project_name = p['name'].split('/', 1)[-1]
#            actions.append(Action(
#                name="%s: %s" % (p['name'], t['title']),
#                tags=tags,
#                duration=ACTION_TIMES['couple of days'],
#                provider="easyfix",
#                extra_data={"ticket_url": t['url'], 'project_org': project_org, 'project_name': project_name, 'project_fullname': p['name'], 'languages': tags[:-1]},
#                )
#              )
#            break
#
#    with open(EASYFIX_CACHE_PROJECTINFO, 'w') as f:
#        json.dump(project_info, f)
#
#    return actions
#

if __name__ == "__main__":
    import pprint
    pprint.pprint(get_actions())
#    project_info = {}
#    # FIXME
#    statsfile = "./oraculum_easyfix_cache.json"
#    if len(sys.argv) > 1 and os.path.exists(sys.argv[1]):
#        statsfile = sys.argv[1]
#    if os.path.isfile(statsfile):
#        with open(statsfile) as f:
#            project_info = json.load(f)
#
#    projects = parse_easyfix_page()
#    for p in projects:
#        print(p['name'], p['url'])
#        if p['url'] in project_info:
#            p.update(project_info[p['url']])
#        else:
#            data = grab_project_metadata(p['url'])
#            p.update(data)
#            project_info[p['url']] = data
#
#    # FIXME - kill it with fire
#    with open(statsfile, 'w') as f:
#        f.write("{\n")
#        for p in project_info:
#            f.write('  "%s":{\n' % p)
#            for k in project_info[p]:
#                f.write('    "%s": %s,\n' % (k, json.dumps(project_info[p][k])))
#            f.seek(f.tell()-2)
#            f.write('\n  },\n')
#        f.seek(f.tell()-2)
#        f.write('\n}\n')
