import re
import requests

from oraculum.utils import Action, ACTION_TIMES
from oraculum.utils import schedule

INFO = {
        'provider': 'fedora_manual_testing',
        'name': 'Fedora Release validation',
        'duration': ACTION_TIMES['few hours'],
        'description': 'The goal of release validation testing is to ensure the release candidate meets the requirements for Fedora releases. You will manually execute test cases to verify installation and basic functionality of the various Fedora products.',
        'scope': 'Testing',
        'tags': False,
       }

RE_HTML = re.compile(r'</?.*?>')
RE_WIKI_ANCHOR = re.compile('[^a-zA-Z0-9_.:-]')

def wiki_anchor_replace(tcname):
    def replace_method(match):
        symbol = match.group(0)
        return '_' if symbol == ' ' else '.%s' % hex(ord(symbol))[2:].upper()
    tcname = RE_HTML.sub('', tcname.strip())
    return RE_WIKI_ANCHOR.sub(replace_method, tcname)


import pprint

def load_stats():
    resp = requests.get("https://www.happyassassin.net/testcase_stats/%d/data.json" % schedule.current_devel())
    if not resp.ok:
        return []
    data = resp.json()

    composes_rev = list(reversed(data['allcomposes']))
    composes_len = len(composes_rev)

    selection = []
    threshold = 5 # FIXME - use better scoring
    if len(composes_rev) <= 5:
        threshold = 1
    for testtype in data['stats']:
        # FIXME: I did not find the security lab ISO in the ISOs section on the wiki page and it seems like
        #  this should test the specific spin. Get more info in the future, skipping for now.
        if testtype.strip().lower() == 'security lab':
            continue

        for section in data['stats'][testtype]:
            for tc in data['stats'][testtype][section]:
                if tc['inactive']:
                    continue
                envs = {}
                for env, last_tested in tc['last_tested_by_env'].items():
                    try:
                        score = composes_rev.index(last_tested)
                    except ValueError:
                        score = composes_len
                    if score >= threshold:
                        envs[env] = score
                if envs:
                    tc['name'] = tc['testcase_name']
                    tc['display_name'] = tc['testcase_display_name']
                    tc['score'] = score
                    tc['section'] = section
                    tc['testtype'] = testtype
                    tc['envs'] = [k for k in sorted(envs, key=envs.get, reverse=True)]
                    selection.append(tc)

    return selection

def get_actions():
    actions = []
    for tc in load_stats():
        if tc['milestone'] == "Optional" or any([b in tc['section'].lower() for b in ["non-blocking", "non release-blocking"]]):
            continue
        # FIXME: filter testcases by relevance
        #  (e.g. prefer Beta milestone when in Beta phase instead of Final milestone)

        # some nice-ification for the tags and user-shown data
        tc['section'] = RE_HTML.sub('', tc['section'])
        tc_name_human = tc['name'].replace('QA:Testcase', '').replace('_', ' ').strip().split(' ')
        tc_name_human[0] = tc_name_human[0].capitalize()
        tc_name_human = ' '.join(tc_name_human)
        if tc['display_name']:
            tc_name_human = "%s - %s" % (tc_name_human, tc['display_name'])
        else:
            tc['display_name'] = tc['name']

        actions.append(Action(
                name=tc_name_human,
                tags=[],
                duration=INFO['duration'],
                provider=INFO['provider'],
                extra_data={
                    'testcase_url': tc['testcase_url'],
                    'milestone': tc['milestone'],
                    'section': tc['section'],
                    'testtype': tc['testtype'],
                    'name': tc['name'],
                    'display_name': tc['display_name'],
                    'envs': tc['envs'],
                    'matrix_url': 'https://fedoraproject.org/wiki/Test_Results:Current_%s_Test#%s' % (tc['testtype'], wiki_anchor_replace(tc['section'])),
                  }
              )
            )
    return actions

if __name__ == "__main__":
    import pprint
    data = get_actions()
    pprint.pprint(data, width=200)
    print(len(data))
